/*
  First things first, all the command parsing is done using R.Unger's
  awesome commanParser

  This is the actual project for cs644
*/

import java.awt.*;
import java.awt.Checkbox;
import java.awt.event.*;
import java.applet.Applet;
import java.util.Vector;
import java.lang.Thread;

public class Project extends Applet implements ActionListener,ItemListener
{

  public Graph wireFrame;
  public Vector faceNormals;
  public CommandParser parse;
  public TextField commInput;  // for reading off the applet
  public TextField numViews;   // for choosing different views of projection
  public TextArea display;


  private Rotate rot = null;

  private MyCanvas myCanvas;
  private Panel p1;
  private Choice choices;
  private Choice choiceViews;
  private Button goButton;
  private Button rotate;
  private Button stopRotate;
  private Button pauseRotate;
  //private CheckboxGroup views;
  //private Checkbox bestView;
  //private Checkbox worstView;

  //  private int view = 1;

  private int numBestViews = 0;

  static final int CUBE = 0;
  static final int HOUSE = 1;
  static final int PYRAMID = 2;
  static final int STAR = 3;
  static final int DIAMOND = 4;
  static final int PINS = 5;

  private int lookingAt = -1;

  public void init() 
  {
    parse = new CommandParser(this.getDocumentBase());
    parse.CommandParserInit(null);
    commInput = new TextField(45);  

    // layout of applet
    setSize(600,500);
    setVisible(true);
    GridBagLayout gbl = new GridBagLayout();
    GridBagConstraints gbc = new GridBagConstraints();
    setLayout(gbl);
    gbc.fill = GridBagConstraints.BOTH;
    gbc.weightx = 100;
    gbc.weighty = 0;
    add(commInput,gbl,gbc,0,0,1,1);
   

    display = new TextArea("",10,50,TextArea.SCROLLBARS_VERTICAL_ONLY);
    display.setFont(new Font("Monaco",Font.PLAIN,10));
    display.setForeground(Color.black);
    display.setEditable(false);
    add(display,gbl,gbc,0,1,1,10);
    
    wireFrame = new Graph(parse,display);    

 
    // this class takes care of inserting strings ending
    // in a \n for the command parser
    commInput.addActionListener(this);
    
    // called once, first
    
    // panel setup
    
    p1 = new Panel();
    p1.setLayout(new FlowLayout());

    // choices setup
    
    choices = new Choice();
    //gbl.setConstraints(choices,gbc);    
    choices.add("Cube");
    choices.add("House");
    choices.add("Pyramid");
    choices.add("Diamond");
    choices.add("Pins");
    choices.add("Star");
    choices.addItemListener(this);
    p1.add(choices);


    p1.add(new Label("  Enter View Number:"));
    numViews = new TextField(3);
    numViews.addActionListener(this);
    p1.add(numViews);


    // button to start computations
    
    goButton = new Button("Compute");
    //gbl.setConstraints(goButton,gbc);
    goButton.addActionListener(this);
    p1.add(goButton);

    rotate = new Button("Rotate");
    rotate.addActionListener(this);
    p1.add(rotate);

    stopRotate = new Button("Stop");
    stopRotate.addActionListener(this);
    p1.add(stopRotate);

    pauseRotate = new Button("Pause");
    pauseRotate.addActionListener(this);
    p1.add(pauseRotate);

    // add the panel finally
    gbl.setConstraints(p1,gbc);
    add(p1,gbl,gbc,0,11,1,1);

    
    // canvas setup
    myCanvas = new MyCanvas();
    gbc.weighty = 100;
    add(myCanvas,gbl,gbc,0,12,1,1);

  }
  
  public void add(Component c, GridBagLayout gbl, GridBagConstraints
		  gbc, int x, int y, int w, int h)
  {
    gbc.gridx = x;
    gbc.gridy = y;
    gbc.gridwidth = w;
    gbc.gridheight = h;
    gbl.setConstraints(c,gbc);
    add(c);
  }


  public void start()
  {
    // called to start execution
    myCanvas.init();
    repaint();
  }
  
  public void stop()
  {
    // called when applet should stop execution
  }

  public void paint( Graphics g ) 
  {
    // put code to draw applet in here
  }


  // deal with awt evts from string inputs 
  public void actionPerformed(ActionEvent e)
  {

    // when event is from the text field
    if (e.getSource() == commInput)
      {
	parse.SendCommand(commInput.getText());
	commInput.setText("");  // make it empty again
      }

    if (e.getSource() == numViews)
      {
	if (lookingAt == -1)
	  {
	    display.append("CHOOSE A SHAPE FIRST\n");
	    numViews.setText("");
	    return;
	  }
	
	String inputted = numViews.getText();
	int numv = Integer.valueOf(inputted).intValue();
	if (numv >= numBestViews)
	  {
	    display.append("ONLY "+numBestViews+" VIEWS AVAILABLE, TRY AGAIN\n");
	    numViews.setText("");
	    return;
	  }

	if (numv < 0)
	  {
	    numViews.setText("");
	    return;
	  }

	showView(numv);
	numViews.setText("");
      }

    // deal with the button
    if (e.getSource() == goButton)
      {
	display.append("Computing....\n");
	if (wireFrame == null)
	  return;
	String stuffread = wireFrame.toString();
	display.append("The following was read from file:");
	display.append(stuffread+"\n");

	if (wireFrame.edges.size() < 1)
	  {
	    display.append("No edges in file,...can't do anything\n");
	    return;
	  }

	wireFrame.averageNode();
	String avgnodestring = wireFrame.avgNode.toString();
	display.append("Average node computed is: "+avgnodestring+"\n");
	display.append("Computing normals and view angle....This takes a while....patience\n");
	wireFrame.computeNormals();
	display.append("The normals computed are:\n");
	for (int i=0;i<wireFrame.normals.size();i++)
	  {
	    display.append("Normal "+i+": ");
	    display.append(wireFrame.normals.elementAt(i).toString());	    
	    display.append("\n");
	  }
	display.append("Computing view angle....This takes a while\n");	    
	display.append("Best viewing vector = "+wireFrame.bestVect);
	myCanvas.initGraph(wireFrame);
	myCanvas.updateCanvas();	
      }

    if (e.getSource() == rotate)
      {
	display.append("Rotating...\n");
	if (wireFrame == null || wireFrame.avgNode == null){
	  display.append("Nothing to rotate\n");
	  return;
	}
	if (rot == null)
	  {
	    rot = new Rotate(wireFrame,myCanvas);
	    rot.start();
	  }
	else if (rot.isAlive())
	  {
	    rot.resume();
	  }
	else
	  {
	    display.append("Rotation already in progress\n");
	    return;
	  }	  
      }

    if (e.getSource() == stopRotate)
      {
	display.append("Stoping Rotating...\n");
	if (wireFrame == null){
	  display.append("Nothing to stop\n");
	  return;
	}
	if (rot == null)
	  {
	    display.append("No rotation in progress\n");
	    return;
	  }
	rot.stop();
	rot = null;
      }

    if (e.getSource() == pauseRotate)
      {
	display.append("Pausing Rotation...\n");
	if (wireFrame == null){
	  display.append("Nothing to stop\n");
	  return;
	}
	if (rot == null)
	  {
	    display.append("No rotation in progress\n");
	    return;
	  }
	rot.suspend();
      }

  }


  public void itemStateChanged(ItemEvent e)
  {    

    if (e.getSource() instanceof Choice)
      {
	Choice temp = (Choice)e.getSource();
	if (temp.getSelectedItem().equals("Cube"))
	  {
	    display.append("Cube...\n");
	    wireFrame.clearGraph();
	    parse.SendCommand("run cube.demo");	    
	    String stuffread = wireFrame.toString();
	    display.append("The following was read from file:");
	    display.append(stuffread+"\n");
	    wireFrame.averageNode();
	    String avgnodestring = wireFrame.avgNode.toString();
	    display.append("Average node computed is: "+avgnodestring+"\n");
	    display.append("Computing normals and view angle....This takes a while....patience\n");
	    wireFrame.computeNormals();
	    display.append("The normals computed are:\n");
	    for (int i=0;i<wireFrame.normals.size();i++)
	      {
		display.append("Normal "+i+": ");
		display.append(wireFrame.normals.elementAt(i).toString());	    
		display.append("\n");
	      }
	    display.append("Computing view angle....This takes a while\n");
	    display.append("Best viewing vector = "+wireFrame.bestVect+"\n");
	    myCanvas.initGraph(wireFrame);
	    myCanvas.updateCanvas();
	    numBestViews = wireFrame.getNumBestVectors();
	    display.append("There are "+numBestViews+" available views\n");
	    lookingAt = CUBE;
	  }
	else if (temp.getSelectedItem().equals("Pyramid"))
	  {
	    display.append("Pyramid...\n");
	    wireFrame.clearGraph();
	    parse.SendCommand("run pyramid.demo");
	    String stuffread = wireFrame.toString();
	    display.append("The following was read from file:");
	    display.append(stuffread+"\n");
	    wireFrame.averageNode();
	    String avgnodestring = wireFrame.avgNode.toString();
	    display.append("Average node computed is: "+avgnodestring+"\n");
	    display.append("Computing normals and view angle....This takes a while...patience\n");
	    wireFrame.computeNormals();
	    display.append("The normals computed are:\n");
	    for (int i=0;i<wireFrame.normals.size();i++)
	      {
		display.append("Normal "+i+": ");
		display.append(wireFrame.normals.elementAt(i).toString());	    
		display.append("\n");
	      }
	    display.append("Best viewing vector = "+wireFrame.bestVect+"\n");
	    myCanvas.initGraph(wireFrame);
	    myCanvas.updateCanvas();
	    numBestViews = wireFrame.getNumBestVectors();
	    display.append("There are "+numBestViews+" available views\n");
	    lookingAt = PYRAMID;
	  }
	else if (temp.getSelectedItem().equals("Diamond"))
	  {
	    display.append("Diamond...\n");
	    wireFrame.clearGraph();
	    parse.SendCommand("run diamond.demo");
	    String stuffread = wireFrame.toString();
	    display.append("The following was read from file:");
	    display.append(stuffread+"\n");
	    wireFrame.averageNode();
	    String avgnodestring = wireFrame.avgNode.toString();
	    display.append("Average node computed is: "+avgnodestring+"\n");
	    display.append("Computing normals and view angle....This takes a while...patience\n");
	    wireFrame.computeNormals();
	    display.append("The normals computed are:\n");
	    for (int i=0;i<wireFrame.normals.size();i++)
	      {
		display.append("Normal "+i+": ");
		display.append(wireFrame.normals.elementAt(i).toString());	    
		display.append("\n");
	      }
	    display.append("Computing normals and view angle....This takes a while...patience\n");
	    display.append("Best viewing vector = "+wireFrame.bestVect+"\n");
	    myCanvas.initGraph(wireFrame);
	    myCanvas.updateCanvas();
	    numBestViews = wireFrame.getNumBestVectors();
	    display.append("There are "+numBestViews+" available views\n");
	    lookingAt = DIAMOND;
	  }
	else if (temp.getSelectedItem().equals("Star"))
	  {
	    display.append("Star...\n");
	    wireFrame.clearGraph();
	    parse.SendCommand("run star.demo");
	    String stuffread = wireFrame.toString();
	    display.append("The following was read from file:");
	    display.append(stuffread+"\n");
	    wireFrame.averageNode();
	    String avgnodestring = wireFrame.avgNode.toString();
	    display.append("Average node computed is: "+avgnodestring+"\n");
	    display.append("Computing normals and view angle....This takes a while....patience\n");
	    wireFrame.computeNormals();
	    display.append("The normals computed are:\n");
	    for (int i=0;i<wireFrame.normals.size();i++)
	      {
		display.append("Normal "+i+": ");
		display.append(wireFrame.normals.elementAt(i).toString());	    
		display.append("\n");
	      }
	    display.append("Best viewing vector = "+wireFrame.bestVect+"\n");
	    myCanvas.initGraph(wireFrame);
	    myCanvas.updateCanvas();
	    numBestViews = wireFrame.getNumBestVectors();
	    display.append("There are "+numBestViews+" available views\n");
	    lookingAt = STAR;
	  }
	else if (temp.getSelectedItem().equals("House"))
	  {
	    display.append("House...\n");
	    wireFrame.clearGraph();
	    parse.SendCommand("run house.demo");
	    String stuffread = wireFrame.toString();
	    display.append("The following was read from file:");
	    display.append(stuffread+"\n");
	    wireFrame.averageNode();
	    String avgnodestring = wireFrame.avgNode.toString();
	    display.append("Average node computed is:"+avgnodestring+"\n");
	    display.append("Computing normals and view angle....This takes a while....patience\n");
	    wireFrame.computeNormals();
	    display.append("The normals computed are:\n");
	    for (int i=0;i<wireFrame.normals.size();i++)
	      {
		display.append("Normal "+i+": ");
		display.append(wireFrame.normals.elementAt(i).toString());	    
		display.append("\n");
	      }
	    display.append("Best viewing vector = "+wireFrame.bestVect+"\n");
	    myCanvas.initGraph(wireFrame);
	    myCanvas.updateCanvas();
	    numBestViews = wireFrame.getNumBestVectors();
	    display.append("There are "+numBestViews+" available views\n");
	    lookingAt = STAR;
	  }
	else if (temp.getSelectedItem().equals("Pins"))
	  {
	    display.append("Pins...\n");
	    wireFrame.clearGraph();
	    parse.SendCommand("run pins.demo");
	    String stuffread = wireFrame.toString();
	    display.append("The following was read from file:");
	    display.append(stuffread+"\n");
	    wireFrame.averageNode();
	    String avgnodestring = wireFrame.avgNode.toString();
	    display.append("Average node computed is:"+avgnodestring+"\n");
	    display.append("Computing normals and view angle....This takes a while....patience\n");
	    wireFrame.computeNormals();
	    display.append("The normals computed are:\n");
	    for (int i=0;i<wireFrame.normals.size();i++)
	      {
		display.append("Normal "+i+": ");
		display.append(wireFrame.normals.elementAt(i).toString());	    
		display.append("\n");
	      }
	    display.append("Best viewing vector = "+wireFrame.bestVect+"\n");
	    myCanvas.initGraph(wireFrame);
	    myCanvas.updateCanvas();
	    numBestViews = wireFrame.getNumBestVectors();
	    display.append("There are "+numBestViews+" available views\n");
	    lookingAt = PINS;
	  }

      }     
  }




public void showView(int v)
  {
    if (wireFrame.avgNode == null)
      {
	display.append("CHOOSE A SHAPE FIRST\n");
	return;
      }
    wireFrame.showView(v);
    myCanvas.initGraph(wireFrame);
    myCanvas.updateCanvas();	    
  }


}







